#define BOOST_TEST_MODULE "testcase for HarkConfigFileXMLReader class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <boost/shared_ptr.hpp>
#include <boost/format.hpp>

#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/harkconfig/TSPListFile.hpp>
#include <harkio2/harkconfig/MicArrayLocationFile.hpp>
#include <harkio2/harkconfig/NoiseLocationFile.hpp>
#include <harkio2/harkconfig/HarkConfigFileXMLReader.hpp>
#include <harkio2/AnalyzeError.hpp>

#include "rangecheck.hpp"

using namespace boost;
using namespace harkio2;
using namespace harkio2::harkconfig;

BOOST_AUTO_TEST_CASE(read_impulseresponse_test)
{
    boost::shared_ptr<HarkConfigFileXMLReader> reader(new HarkConfigFileXMLReader("sample_ir20.xml"));
    BOOST_CHECK(reader != 0);
    boost::shared_ptr<HarkConfigFile> data;

    try
    {
        data = reader->Read();
    }
    catch (const harkio2::AnalyzeError& e)
    {
        BOOST_ERROR((format("%d: %s") % e.line() % e.what()).str());
    }

    BOOST_CHECK(data != 0);

    if (data)
    {
        BOOST_CHECK_EQUAL(data->GetId(), "ImpulseResponseListFile");
        boost::shared_ptr<ImpulseResponseListFile> ir = dynamic_pointer_cast<ImpulseResponseListFile>(data);
        BOOST_CHECK(ir != 0);

        if (ir != 0)
        {
            BOOST_CHECK(ir->GetVersion() == Version(2, 0));
            boost::shared_ptr<ImpulseResponseListFile2_0> ir20 = dynamic_pointer_cast<ImpulseResponseListFile2_0>(ir);
            BOOST_CHECK(ir20 != 0);

            if (ir20 != 0)
            {
                BOOST_CHECK_EQUAL(ir20->GetNumMic(), 2);
                BOOST_CHECK_EQUAL(ir20->GetSourceFileType(), harkio2::FileType::FLOAT);
                BOOST_CHECK_EQUAL(ir20->GetComment(), u8"Hello, world");

                BOOST_CHECK(ir20->GetSources().size() == 1 &&
                        ir20->GetSources()[0].GetFiles().size() == 1 &&
                        ir20->GetSources()[0].GetFiles()[0].GetName() == u8"sample.flt");
            }
        }
    }

}

BOOST_AUTO_TEST_CASE(read_tsp_test)
{
    boost::shared_ptr<HarkConfigFileXMLReader> reader(new HarkConfigFileXMLReader("sample_tsp20.xml"));
    BOOST_CHECK(reader != 0);
    boost::shared_ptr<HarkConfigFile> data;

    try
    {
        data = reader->Read();
    }
    catch (const harkio2::AnalyzeError& e)
    {
        BOOST_ERROR((format("%d: %s") % e.line() % e.what()).str());
    }

    BOOST_CHECK(data != 0);

    if (data)
    {
        BOOST_CHECK_EQUAL(data->GetId(), "TSPListFile");
        boost::shared_ptr<TSPListFile> tsp = dynamic_pointer_cast<TSPListFile>(data);
        BOOST_CHECK(tsp != 0);

        if (tsp != 0)
        {
            BOOST_CHECK(tsp->GetVersion() == Version(2, 0));
            boost::shared_ptr<TSPListFile2_0> tsp20 = dynamic_pointer_cast<TSPListFile2_0>(tsp);
            BOOST_CHECK(tsp20 != 0);

            if (tsp20 != 0)
            {
                BOOST_CHECK_EQUAL(tsp20->GetSynchronousAverage(), 16);
                BOOST_CHECK(tsp20->GetOriginalImpulseFile() ==
                        harkio2::TypedFilename(u8"foo.tsp", harkio2::FileType::FLOAT));
                BOOST_CHECK_EQUAL(tsp20->GetTSPOffset(), 16384);
                BOOST_CHECK_EQUAL(tsp20->GetTSPLength(), 1024);
                BOOST_CHECK_EQUAL(tsp20->GetSignalMax(), 32768);
                BOOST_CHECK_EQUAL(tsp20->GetNumMic(), 2);
                BOOST_CHECK_EQUAL(tsp20->GetSourceFileType(), harkio2::FileType::WAVE);
                BOOST_CHECK_EQUAL(tsp20->GetComment(), u8"今日は, TSP");

                BOOST_CHECK_EQUAL(tsp20->GetSources().size(), 1);

                if (tsp20->GetSources().size() == 1)
                {
                    const harkio2::harkconfig::Source2& source =
                        tsp20->GetSources()[0];

                    BOOST_CHECK(rangecheck::CheckRange<float>(source.GetPosition().GetAngles().first, M_PI * 60 / 180, 0.01));
                    BOOST_CHECK(source.GetFiles().size() == 1 &&
                            source.GetFiles()[0].GetName() == u8"bar.wav");
                }
            }
        }

    }

}

BOOST_AUTO_TEST_CASE(read_mic_test)
{
    boost::shared_ptr<HarkConfigFileXMLReader> reader(new HarkConfigFileXMLReader("sample_mic20.xml"));
    BOOST_REQUIRE(reader != 0);
    boost::shared_ptr<HarkConfigFile> data;

    try
    {
        data = reader->Read();
    }
    catch (const harkio2::AnalyzeError& e)
    {
        BOOST_ERROR((format("%d: %s") % e.line() % e.what()).str());
    }

    BOOST_REQUIRE(data != 0);

    BOOST_CHECK_EQUAL(data->GetId(), "MicArrayLocationFile");
    boost::shared_ptr<MicArrayLocationFile> mic = dynamic_pointer_cast<MicArrayLocationFile>(data);
    BOOST_REQUIRE(mic != 0);

    BOOST_CHECK(mic->GetVersion() == Version(2, 0));
    boost::shared_ptr<MicArrayLocationFile2_0> mic20 = dynamic_pointer_cast<MicArrayLocationFile2_0>(mic);
    BOOST_REQUIRE(mic20 != 0);

    BOOST_CHECK_EQUAL(mic20->GetComment(), u8"マイク位置");

    BOOST_REQUIRE_EQUAL(mic20->GetMics().size(), 1);
    const harkio2::harkconfig::Mic1& mic1 = mic20->GetMics()[0];
    BOOST_CHECK_CLOSE(mic1.GetPosition().GetLength(), 4.4, 1); // checks length is 4.4  (+/- 1% range)
    BOOST_CHECK_EQUAL(mic1.GetComment(), u8"マイク0");

}

BOOST_AUTO_TEST_CASE(read_noise_test)
{
    boost::shared_ptr<HarkConfigFileXMLReader> reader(new HarkConfigFileXMLReader("sample_noise20.xml"));
    BOOST_REQUIRE(reader != 0);
    boost::shared_ptr<HarkConfigFile> data;

    try
    {
        data = reader->Read();
    }
    catch (const harkio2::AnalyzeError& e)
    {
        BOOST_ERROR((format("%d: %s") % e.line() % e.what()).str());
    }

    BOOST_REQUIRE(data != 0);

    BOOST_CHECK_EQUAL(data->GetId(), "NoiseLocationFile");
    boost::shared_ptr<NoiseLocationFile> noise = dynamic_pointer_cast<NoiseLocationFile>(data);
    BOOST_REQUIRE(noise != 0);

    BOOST_CHECK(noise->GetVersion() == Version(2, 0));
    boost::shared_ptr<NoiseLocationFile2_0> noise20 = dynamic_pointer_cast<NoiseLocationFile2_0>(noise);
    BOOST_REQUIRE(noise20 != 0);

    BOOST_CHECK_EQUAL(noise20->GetComment(), u8"ノイズ方向");

    BOOST_REQUIRE_EQUAL(noise20->GetNoiseSources().size(), 1);
    const harkio2::harkconfig::NoiseSource1& noise1 = noise20->GetNoiseSources()[0];
    BOOST_CHECK(noise1.GetPosition().IsDirection());
    BOOST_CHECK_EQUAL(noise1.GetComment(), u8"ノイズ0");

}

